﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class CollisionManager : SingletonMonoBehaviour<CollisionManager>
{
    private List<IngameEntityPoint> points;
    private List<IIngameEntity> ingameEntities;

    private void Awake()
    {
        points = new List<IngameEntityPoint>();
        ingameEntities = new List<IIngameEntity>();
    }

    public void RegisterEntity(IIngameEntity ingameEntity)
    {
        ingameEntities.Add(ingameEntity);
    }

    public void RegisterPoint(IngameEntityPoint ingameEntityPoint)
    {
        points.Add(ingameEntityPoint);
    }

    public void UnregisterEntityAndPoints(IIngameEntity ingameEntity)
    {
        ingameEntities.Remove(ingameEntity);

        var registeredEntityPoints = points.Where(point => point.TargetEntity == ingameEntity).ToList();

        foreach (var point in registeredEntityPoints)
        {
            points.Remove(point);
        }
    }

    public List<IIngameEntity> CheckCollision(Vector2 previousWorldPosition, Vector2 currentWorldPosition)
    {
        List<IIngameEntity> result = null;
        foreach (var entity in ingameEntities)
        {
            if (entity.CheckCollision(previousWorldPosition, currentWorldPosition))
            {
                if (result == null)
                    result = new List<IIngameEntity>();

                result.Add(entity);
            }
        }

        return result;
    }
}
