﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using UnityEngine;

public class PoolManager : SingletonMonoBehaviour<PoolManager>
{
    private Dictionary<string, List<GameObject>> pools;

    private void Awake()
    {
        pools = new Dictionary<string, List<GameObject>>();
    }

    public GameObject InstantiateOrGetNormal(GameObject prefab, Action<GameObject> instantiate)
    {
        return InstantiateOrGet(prefab, instantiate);
    }

    private GameObject InstantiateOrGet(GameObject prefab, Action<GameObject> instantiate)
    {
        var identifier = GetIdentifier(prefab);

        GameObject instance = null;

        if (pools.ContainsKey(identifier))
        {
            var pool = pools[identifier];
            if (pool.Count > 0)
            {
                instance = pool[pool.Count - 1];
                pool.RemoveAt(pool.Count - 1);

                instance.transform.parent = null;
                instance.transform.CopyPositionAndRotatationFrom(prefab.transform);

                if (instantiate != null)
                    instantiate(instance);

                instance.SetActive(true);
                instance.SendMessage("OnPoolReactivate", SendMessageOptions.DontRequireReceiver);
            }
        }
        else
        {
            // Register
            pools[identifier] = new List<GameObject>();
        }

        if (instance == null)
        {
            instance = UnityHelper.InstantiateGameObject(prefab);
            if (instantiate != null)
                instantiate(instance);
        }

        return instance;
    }

    public bool IsRegistered(GameObject instance)
    {
        var identifier = GetIdentifier(instance);
        return pools.ContainsKey(identifier);
    }

    public void AddToPool(GameObject instance)
    {
        var identifier = GetIdentifier(instance);
        pools[identifier].Add(instance);
        instance.SetActive(false);
        instance.transform.parent = transform;
    }

    public string GetIdentifier(GameObject gameObject)
    {
        return gameObject.GetNameWithoutClone();
    }

    public T InstantiateOrGetNormal<T>(T prefab, Action<GameObject> instantiate) where T : Component
    {
        return InstantiateOrGetNormal(prefab.gameObject, instantiate).GetComponent<T>();
    }

    public string GetIdentifier<T>(T component) where T : Component
    {
        return GetIdentifier(component.gameObject);
    }
}
