using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Settings : PersistentSingletonMonoBehaviour<Settings>
{
    public event Action<bool> EventMusicChanged;
    public event Action<bool> EventSfxChanged;

    [SerializeField] private float infoFontSize;
    [SerializeField] private float infoDisplayTime;

    [SerializeField] private bool saveSettings;
    [SerializeField] private bool music;
    [SerializeField] private bool sfx;

    private VisualSettings visualSettings;
    private string infoMessage;
    private Countdown infoCountdown;

    protected override void OnFirstLoad()
    {
        infoCountdown = new Countdown(infoDisplayTime);
    }

    protected override void OnFirstLoadOrSwitch()
    {
        visualSettings = VisualSettings.Instance;

        if (saveSettings)
        {
            music = UnityHelper.PlayerPrefsGetBool("Music", Music);
            sfx = UnityHelper.PlayerPrefsGetBool("Sfx", Sfx);
        }
    }

    private void Update()
    {
        if (Input.GetKeyDown(KeyCode.M))
        {
            Music = !Music;
            SetInfoMessage("Music: " + (Music ? "On" : "Off"));
        }

        if (Input.GetKeyDown(KeyCode.N))
        {
            Sfx = !Sfx;
            SetInfoMessage("Sound Effects: " + (Sfx ? "On" : "Off"));
        }

        if (Input.GetKeyDown(KeyCode.F))
        {
            Screen.fullScreen = !Screen.fullScreen;
        }

        if (infoMessage != null)
        {
            if (infoCountdown.Update())
            {
                infoMessage = null;
            }
        }
    }

    private void OnGUI()
    {
        if (infoMessage == null)
            return;

        UnityHelper.SetFont(visualSettings.FontDefault, infoFontSize);
        UnityHelper.ShowText(infoMessage, new Vector2(Screen.width / 2f, Screen.height / 2f), UnityHelper.AnchorCenter, Color.white, TextAlignment.Center);
    }

    private void SetInfoMessage(string message)
    {
        infoMessage = message;
        infoCountdown.Reset();
    }

    public bool Music
    {
        get { return music; }
        set
        {
            if (music == value)
                return;

            music = value;

            UnityHelper.PlayerPrefsSetBool("Music", Music);

            if (EventMusicChanged != null)
                EventMusicChanged(music);
        }
    }

    public bool Sfx
    {
        get { return sfx; }
        set
        {
            if (sfx == value)
                return;

            sfx = value;

            UnityHelper.PlayerPrefsSetBool("Sfx", Sfx);

            if (EventSfxChanged != null)
                EventSfxChanged(sfx);
        }
    }
}
