﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class IngameEntityPoint
{
    public class NetData
    {
        public IngameEntityPoint PreviousNetPoint { get; set; }
        public IngameEntityPoint NextNetPoint { get; set; }

        public bool IsEndpoint { get { return (PreviousNetPoint == null) || (NextNetPoint == null); } }

        public NetData(IngameEntityPoint previousNetPoint, IngameEntityPoint nextNetPoint)
        {
            PreviousNetPoint = previousNetPoint;
            NextNetPoint = nextNetPoint;
        }

        public bool IsConnectedTo(IngameEntityPoint point)
        {
            return (PreviousNetPoint == point) || (NextNetPoint == point);
        }

        public bool IsNotConnectedTo(IngameEntityPoint point)
        {
            return !IsConnectedTo(point);
        }
    }

    public IIngameEntity TargetEntity { get; private set; }
    private int shapeIndex;
    private int pointIndex;

    public Vector2 LastProcessedWorldPosition { get; set; }

    public IngameEntityPoint(IIngameEntity targetEntity, int shapeIndex, int pointIndex)
    {
        this.TargetEntity = targetEntity;
        this.shapeIndex = shapeIndex;
        this.pointIndex = pointIndex;

        LastProcessedWorldPosition = WorldPosition;
    }

    public Vector2 WorldPosition
    {
        get { return TargetEntity.GetPointWorldPosition(shapeIndex, pointIndex); }
    }

    public bool Compare(IIngameEntity targetEntity, int shapeIndex, int pointIndex)
    {
        return (TargetEntity == targetEntity) && (this.shapeIndex == shapeIndex) && (this.pointIndex == pointIndex);
    }
}