﻿using System;
using UnityEngine;
using Random = UnityEngine.Random;

[Serializable]
public class ScreenShake
{
    [SerializeField] private float duration;
    [SerializeField] private float power;
    [SerializeField] private float powerToAngleConversion = 10f;
    [SerializeField] private EaseType powerEase = EaseType.Linear;

    public float Power
    {
        get { return power; }
        set { power = value; }
    }

    public float Duration { get { return duration; } }

    public Vector3 CurrentOffset
    {
        get { return Active ? currentOffset : Vector3.zero; }
        private set { currentOffset = value; }
    }

    private float countdown = 0;
    private Vector3 currentOffset;
    private Quaternion currentRotation;

    public void Update()
    {
        if (Time.timeScale == 0)
            return;

        if (countdown > 0)
        {
            countdown -= UnityHelper.RealDeltaTime;

            if (countdown <= 0)
            {
                countdown = 0;
                CurrentOffset = Vector3.zero;
            }
            else
            {
                UpdateOffset();
            }
        }
    }

    public void StartShake()
    {
        StartShake(duration);
    }

    public void StartShake(float newDuration)
    {
        duration = newDuration;
        countdown = newDuration;
        UpdateOffset();
    }

    private void UpdateOffset()
    {
        var currentPower = CurrentPower;

        //CurrentOffset = Random.onUnitSphere * Random.Range(0, power);
        CurrentOffset = Random.insideUnitCircle * currentPower;

        var rotationPower = currentPower * powerToAngleConversion;
        CurrentRotation = Quaternion.Euler(0, 0, Random.Range(-rotationPower, rotationPower));
    }

    public Quaternion CurrentRotation
    {
        get { return Active ? currentRotation : Quaternion.identity; }
        set { currentRotation = value; }
    }

    public bool HasPower { get { return power > 0; } }
    public float CurrentPower { get { return Active ? (Ease.ChangeFloat(powerEase, PercentLeft) * power) : 0f; } }
    public float PercentLeft { get { return Active ? (countdown / duration) : 0; } }
    public bool Active { get { return countdown > 0; } }
}
