﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class ShapeRegularPolygon : AbstractShape
{
    public enum AngleTreatmentType
    {
        AnglesWeightedPerSequence,
        AdditiveAngles,
        AbsoluteAngles
    }

    [Serializable]
    public class PointData
    {
        public float distance = 1;
        public float anglePercentFactor = 1;
        public float anglePercentDivider = 1;
        public float controlPointDistance = 1;
        public float controlPointAngle = 0;
        public float widthScale = 1f;

        public float AnglePercent { get { return anglePercentFactor / anglePercentDivider; } }
    }

    [SerializeField] private PointData[] sequence;
    [SerializeField] private int sequenceRepetition = 3;
    [SerializeField] private float startAnglePercent;
    [SerializeField] private AngleTreatmentType angleTreatmentType = AngleTreatmentType.AnglesWeightedPerSequence;
    [SerializeField] private float controlPointDistanceMultiplier = 0f;

    public PointData[] Sequence
    {
        get { return sequence; }
    }

    public override void Refresh()
    {
        base.Refresh();

        if ((angleTreatmentType == AngleTreatmentType.AbsoluteAngles) && (sequenceRepetition > 1))
            throw new Exception("Absolute Angles and more than one sequence? That doesn't make ANY sense.");

        int pointCount = sequence.Length * sequenceRepetition;

        RefreshPointCount(pointCount);

        var defaultAngle = 2 * Mathf.PI;

        if (angleTreatmentType == AngleTreatmentType.AnglesWeightedPerSequence)
        {
            var angleSequenceSum = 0f;
            foreach (var point in sequence)
            {
                angleSequenceSum += point.AnglePercent;
            }

            angleSequenceSum *= sequenceRepetition;
            defaultAngle /= angleSequenceSum;
        }

        float angle = startAnglePercent * 2 * Mathf.PI;

        var totalPointIndex = 0;
        for (int sequenceIndex = 0; sequenceIndex < sequenceRepetition; sequenceIndex++)
        {
            for (int sequencePointIndex = 0; sequencePointIndex < sequence.Length; sequencePointIndex++)
            {
                var pointData = sequence[sequencePointIndex];

                if (angleTreatmentType == AngleTreatmentType.AbsoluteAngles)
                {
                    angle = -pointData.AnglePercent * defaultAngle;
                }

			    var position = new Vector3(Mathf.Cos(angle), Mathf.Sin(angle)) * pointData.distance * scale;

                var controlPointAngle = angle + (90 - pointData.controlPointAngle) * Mathf.Deg2Rad;

		        //var innerControlPoint = new Vector3(position.y, -position.x).normalized * pointData.controlPointDistance * scale * controlPointDistanceMultiplier;
                var innerControlPoint = new Vector3(Mathf.Cos(controlPointAngle), Mathf.Sin(controlPointAngle)) * pointData.controlPointDistance * scale * controlPointDistanceMultiplier;
		        //var outerControlPoint = -innerControlPoint;

		        SetNaturalPoint(totalPointIndex, position, innerControlPoint, pointData.widthScale);

                if (angleTreatmentType != AngleTreatmentType.AbsoluteAngles)
                {
                    angle -= pointData.AnglePercent * defaultAngle;
                }

                totalPointIndex++;
            }
        }

        RefreshMesh();
    }
}
