﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class VisualSettings : SingletonMonoBehaviour<VisualSettings>
{
    [SerializeField] private float outlineWidth = 0.05f;
    [SerializeField] private float antiAliasWidth = 0.05f;

    [SerializeField] private Font fontTitle;
    [SerializeField] private Font fontDefault;
    [SerializeField] private bool fontTitleAllUpperCase = false;

    [SerializeField] private Color buttonColorNormal = Color.white;
    [SerializeField] private Color buttonColorHovered = new Color(0.2f, 0.2f, 1f);
    [SerializeField] private Color buttonColorDeactivated = new Color(0.4f, 0.4f, 0.4f);
    [SerializeField] private Color buttonColorMarked = Color.yellow;

    public float OutlineWidth { get { return outlineWidth; } }
    public float AntiAliasWidth { get { return antiAliasWidth; } }

    public Font FontTitle { get { return fontTitle; } }
    public Font FontDefault { get { return fontDefault; } }

    public bool FontTitleAllUpperCase { get { return fontTitleAllUpperCase; } }

    public Color ButtonColorNormal { get { return buttonColorNormal; } }
    public Color ButtonColorHovered { get { return buttonColorHovered; } }
    public Color ButtonColorDeactivated { get { return buttonColorDeactivated; } }
    public Color ButtonColorMarked { get { return buttonColorMarked; } }

    public void RefreshAll()
    {
        var shapes = UnityHelper.FindObjectsOfType<AbstractShape>();
        foreach (var shape in shapes)
        {
            shape.Refresh();
        }
    }

    public void AddGlowToAll()
    {
        var shapes = UnityHelper.FindObjectsOfType<AbstractShape>();
        foreach (var shape in shapes)
        {
            if (shape.Glow)
            {
                if (shape.GetComponentInChildren<GlowCopyParentShape>() != null)
                    continue;

                var glowObject = new GameObject("Glow");

                glowObject.transform.parent = shape.transform;
                glowObject.transform.localPosition = Vector3.zero;

                glowObject.AddComponent<GlowCopyParentShape>();

                var rageSpline = glowObject.GetComponent<RageSpline>();
                rageSpline.SetOutlineColor1(shape.GetComponent<RageSpline>().GetOutlineColor1());
                rageSpline.SetFill(RageSpline.Fill.None);
            }
        }
    }
}
