using System.Linq;
using UnityEngine;
using System.Collections;

[RequireComponent(typeof (TextMesh), typeof(MeshRenderer))]
public class VanishingText : MonoBehaviour
{
    private TextMesh textMesh;

    private float timeLeft;
    private float vanishStartTime;
    private float startAlpha;

    public void Init(string text, Color color, float timeTillVanish, float vanishTime)
    {
        textMesh.text = text;
        textMesh.color = color;
        timeLeft = timeTillVanish + vanishTime;
        vanishStartTime = vanishTime;

        startAlpha = Alpha;
    }

    private void Awake()
    {
        textMesh = GetComponent<TextMesh>();
    }

    private void Start()
    {
    }

    private void Update()
    {
        timeLeft -= Time.smoothDeltaTime;
        if (timeLeft < 0)
        {
            Destroy(gameObject);
        }
        else if (timeLeft < vanishStartTime)
        {
            Alpha = (timeLeft / vanishStartTime) * startAlpha;
        }
    }

    private float Alpha
    {
        get { return renderer.material.color.a; }
        set
        {
            var color = textMesh.color;
            color.a = value;
            textMesh.color = color;
        }
    }
}
